export const BOILERPLATE_OPTIONS = {
  ui: "Extension UI",
  connector: "Connector",
  build: "Build Event Handler",
  edgeFunctions: "Edge Function Injection",
  functions: "Functions Injection",
};

export const example = ({
  boilerplate,
  addEdgeFunctionConfig,
  connectorSupports,
}: {
  boilerplate: string[];
  addEdgeFunctionConfig?: boolean;
  connectorSupports?: { visualEditor: boolean; connect: boolean };
}) => {
  let code = `import { NetlifyExtension } from "@netlify/sdk";

const extension = new NetlifyExtension();
`;

  if (boilerplate.includes(BOILERPLATE_OPTIONS.connector)) {
    code += `const connector = extension.addConnector({
  typePrefix: "Example",
  supports: {
    connect: ${connectorSupports?.connect},
    visualEditor: ${connectorSupports?.visualEditor},
  },
});

/**
 * Define your content models here.
 * https://developers.netlify.com/sdk/api-reference/classes/netlifyconnector/#model
 */
connector.model(async ({ define }) => {
  define.document({
    name: "User",
    fields: {
      name: {
        type: "String",
        required: true,
      },
      posts: {
        type: "Post",
        list: true,
      },
    },
  });

  define.document({
    name: "Post",
    fields: {
      title: {
        type: "String",
        required: true,
      },
      blocks: {
        list: true,
        required: true,
        type: define.object({
          name: "Blocks",
          fields: {
            title: {
              type: "String",
            },
            content: {
              type: "String",
            },
          },
        }),
      },
    },
  });
});

/**
 * Fetch and store data from your API here.
 * https://developers.netlify.com/sdk/api-reference/classes/netlifyconnector/#sync
 */
connector.sync(({ models, isInitialSync }) => {
  switch (true) {
    case isInitialSync: {
      models.User.insert({
        id: "1",
        name: "Annie",
        posts: [
          {
            id: "1",
            __typename: "Post",
          },
        ],

        _status: "published",
        _createdAt: new Date(),
      });

      models.Post.insert({
        id: "1",
        title: "Hello World",
        blocks: [
          {
            title: "Example block title",
            content: "You can create complex content models",
          },
        ],

        _status: "published",
        _createdAt: new Date(),
      });
      break;
    }
    case !isInitialSync: {
      models.User.insert({
        id: "1", // overwrites the existing User node with this ID
        name: "Annie",
        posts: [
          {
            id: "1",
            __typename: "Post",
          },
        ],

        _status: "published",
        _createdAt: new Date(),
      });

      models.Post.insert({
        id: "2", // creates a new Post since this ID doesn't exist yet
        title: "Writing lots of posts these days",
        blocks: [
          {
            title: "Page section",
            content: "what up",
          },
        ],

        _status: "published",
        _createdAt: new Date(),
      });
      break;
    }
  }
});
`;
  }

  if (boilerplate.includes(BOILERPLATE_OPTIONS.build)) {
    code += `
extension.addBuildEventHandler("onPreBuild", () => {
  console.log("Hello there.");
});
  `;
  }

  if (
    addEdgeFunctionConfig ||
    boilerplate.includes(BOILERPLATE_OPTIONS.edgeFunctions)
  ) {
    code += `
extension.addEdgeFunctions("./src/edge-functions", {
  prefix: "ef_prefix",
});
`;
  }

  if (boilerplate.includes(BOILERPLATE_OPTIONS.functions)) {
    code += `
extension.addFunctions("./src/functions", {
  prefix: "my_unique_prefix",
});
`;
  }

  code += `
export { extension };

`;

  return code;
};

export const GITIGNORE = `
.ntli
node_modules
.cache
dev-model.gql
`;

export const createDetails = (extensionName: string) => `
# View and install the ${extensionName} extension.
`;

export const createReadme = (component: string) => `
# Netlify Extension

This extension is created using the [Netlify SDK](https://sdk.netlify.com/get-started/introduction/). It is a boilerplate for creating a new extension.

## Scripts

These are some common scripts you will use when developing your extension. If you want to know what else is possible, [check out the documentation](https://developers.netlify.com/sdk/netlify-sdk-utility-tools-reference/).

### Build

This builds the extension into a \`.ntli\` folder. This is the folder that Netlify uses to run the extension.

\`\`\`bash
npm run build
\`\`\`
${
  component === "connector"
    ? `### Dev

This bundles your extension code and starts a local GraphQL server for any Connectors you have defined. This is useful for testing your extension locally.

\`\`\`bash
npm run dev
\`\`\`
`
    : ""
}
${
  component === "ui"
    ? `### Preview

This bundles your extension code and starts a preview server. This is useful for previewing your extension UI locally.

\`\`\`bash
npm run preview
\`\`\``
    : ""
}
## Publish

Are you ready to deploy and publish your extension? Check out our documentation on [publishing your extension](https://developers.netlify.com/sdk/publish/).
`;

export const testEdgeFunction = `// Documentation: https://sdk.netlify.com

export default () => new Response('Hello, world!');

export const config = {
  path: '/test',
};

`;

export const testFunction = `// Documentation: https://sdk.netlify.com

export default async () => {
  return new Response("Hello, world!");
};

`;
