import { select } from "@inquirer/prompts";
import { resolve } from "path";
import { spawnSync } from "child_process";

import pkg from "fs-extra";
import {
  checkConnectorBoilerplate,
  getCommand,
  isPackageManager,
} from "./utils.js";
import {
  extensionUIDeps,
  extensionUIDevDeps,
  generateUiBoilerplate,
} from "./extension-ui-boilerplate/generate-ui-boilerplate.js";
import { BOILERPLATE_OPTIONS } from "./constants.js";
import chalk from "chalk";

const { ensureDirSync, existsSync } = pkg;

export const uiMigration = async () => {
  const cwd = resolve(process.cwd());
  const srcUiPath = resolve(cwd, "src/ui");
  const isSrcUiFolderExists = existsSync(srcUiPath);
  let boilerplate = [];

  if (isSrcUiFolderExists) {
    const continueWithRename = await select<boolean>({
      message:
        "Your extension already has a `src/ui` folder. We'll rename it to `src/ui-old` and create a new folder. Agree?",
      choices: [
        { name: "Yes", value: true },
        { name: "No", value: false },
      ],
    });
    if (!continueWithRename) {
      console.error("🛑 Stopped creating new Extension UI");
      process.exit(0);
    }
  }

  const hasConnector = await select<boolean>({
    message: "Does your extension have a connector?",
    choices: [
      { name: "Yes", value: true },
      { name: "No", value: false },
    ],
  });

  if (hasConnector) {
    boilerplate.push(BOILERPLATE_OPTIONS.connector);
  }

  const connectorSupports = await checkConnectorBoilerplate(hasConnector);

  const packageManager = await select<string>({
    message: "Which package manager do you use in your project?",
    choices: ["npm", "yarn", "pnpm"].map((choice) => ({
      name: choice,
      value: choice,
    })),
  });

  if (!isPackageManager(packageManager)) {
    console.error("Invalid package manager");
    process.exit(0);
  }

  const deps = [...extensionUIDeps];
  const devDeps = [...extensionUIDevDeps];
  console.log("Adding dependencies...");
  const addCommand = getCommand("add", packageManager).split(" ");
  const addDeps = spawnSync(addCommand[0], [addCommand[1], ...deps], {
    cwd,
    stdio: "inherit",
  });

  if (addDeps.status !== 0) {
    throw new Error(
      `Failed to add dependencies. Please try again with ${packageManager}.`,
    );
  }

  const addDevDeps = spawnSync(
    addCommand[0],
    [addCommand[1], ...devDeps, "-D"],
    {
      cwd,
      stdio: "inherit",
    },
  );

  if (addDevDeps.status !== 0) {
    throw new Error(
      `Failed to add dev dependencies. Please try again with ${packageManager}.`,
    );
  }

  if (isSrcUiFolderExists) {
    console.log("⌨️ renaming `src/ui` to `src/ui-old`...");
    const newSrcUiPath = resolve(cwd, "src/ui-old");
    pkg.renameSync(srcUiPath, newSrcUiPath);
    ensureDirSync(srcUiPath);
    console.log("✅ Successfully renamed `src/ui` to `src/ui-old`");
  }

  console.log("Generating extension ui boilerplate...");

  generateUiBoilerplate(cwd, boilerplate, connectorSupports);
  console.log(
    "\n✅  Find the extension ui files in",
    chalk.bold.cyan("/src/ui"),
  );
};
